﻿using System.Runtime.InteropServices;

namespace TestProject;

/// <summary>
/// 마우스 헬퍼
/// </summary>
public static class MouseHelper
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Import
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 커서 위치 설정하기 - SetCursorPos(x, y)

    /// <summary>
    /// 커서 위치 설정하기
    /// </summary>
    /// <param name="x">X</param>
    /// <param name="y">Y</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32", EntryPoint = "SetCursorPos")]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetCursorPos(int x, int y);

    #endregion
    #region 커서 위치 구하기 - GetCursorPos(mousePoint)

    /// <summary>
    /// 커서 위치 구하기
    /// </summary>
    /// <param name="mousePoint">마우스 포인트</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32")]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetCursorPos(out MousePoint mousePoint);

    #endregion
    #region 마우스 이벤트 발생시키기 - mouse_event(flag, deltaX, deltaY, data, extraInformation)

    /// <summary>
    /// 마우스 이벤트 발생시키기
    /// </summary>
    /// <param name="flag">플래그</param>
    /// <param name="deltaX">델타 X</param>
    /// <param name="deltaY">델타 Y</param>
    /// <param name="data">데이터</param>
    /// <param name="extraInformation">부가 정보</param>
    [DllImport("user32")]
    private static extern void mouse_event(int flag, int deltaX, int deltaY, int data, int extraInformation);

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 커서 위치 설정하기 - SetCursorPosition(mousePoint)

    /// <summary>
    /// 커서 위치 설정하기
    /// </summary>
    /// <param name="mousePoint">마우스 포인트</param>
    public static void SetCursorPosition(MousePoint mousePoint)
    {
        SetCursorPos(mousePoint.X, mousePoint.Y);
    }

    #endregion
    #region 커서 위치 설정하기 - SetCursorPosition(x, y)

    /// <summary>
    /// 커서 위치 설정하기
    /// </summary>
    /// <param name="x">X</param>
    /// <param name="y">Y</param>
    public static void SetCursorPosition(int x, int y)
    {
        SetCursorPos(x, y);
    }

    #endregion
    #region 커서 위치 구하기 - GetCursorPosition()

    /// <summary>
    /// 커서 위치 구하기
    /// </summary>
    /// <returns>마우스 포인트</returns>
    public static MousePoint GetCursorPosition()
    {
        MousePoint mousePoint;

        bool result = GetCursorPos(out mousePoint);

        if(!result)
        {
            mousePoint = new MousePoint(0, 0);
        }

        return mousePoint;
    }

    #endregion
    #region 마우스 이벤트 발생시키기 - MouseEvent(flag)

    /// <summary>
    /// 마우스 이벤트 발생시키기
    /// </summary>
    /// <param name="flag">플래그</param>
    public static void MouseEvent(MouseEventFlags flag)
    {
        MousePoint mousePoint = GetCursorPosition();

        mouse_event((int)flag, mousePoint.X, mousePoint.Y, 0, 0);
    }

    #endregion
}